/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>        /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>       /* HiTech General Include File */
#elif defined(__18CXX)
    #include <p18cxxx.h>   /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)

#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>       /* For true/false definition */

#endif

#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */
#include "fixmath.h"
#include "fixmatrix.h"
/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/

/* i.e. uint8_t <variable_name>; */

/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/

#define _XTAL_FREQ 64000000L
#define __18F26K22

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xc.h>
#include <math.h>
#include <float.h>
#include <plib/i2c.h>
#include <plib/EEP.h>
#include <plib/pconfig.h>


// PIC18F26K22 Configuration Bit Settings

#include <xc.h>

// CONFIG1H
#pragma config FOSC = INTIO67 // Oscillator Selection bits (Internal oscillator block)
#pragma config PLLCFG = OFF // 4X PLL Enable (Oscillator used directly)
#pragma config PRICLKEN = ON // Primary clock enable bit (Primary clock enabled)
#pragma config FCMEN = OFF // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRTEN = ON // Power-up Timer Enable bit (Power up timer enabled)
#pragma config BOREN = ON // Brown-out Reset Enable bits (Brown-out Reset enabled and controlled by software (SBOREN is enabled))
#pragma config BORV = 285 // Brown Out Reset Voltage bits (VBOR set to 2.85 V nominal)

// CONFIG2H
#pragma config WDTEN = ON // Watchdog Timer Enable bits (WDT is always enabled. SWDTEN bit has no effect)
#pragma config WDTPS = 4096 // Watchdog Timer Postscale Select bits (1:2048)

// CONFIG3H
#pragma config CCP2MX = PORTC1 // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF // PORTB A/D Enable bit (PORTB<5:0> pins are configured as digital I/O on Reset)
#pragma config CCP3MX = PORTB5 // P3A/CCP3 Mux bit (P3A/CCP3 input/output is multiplexed with RB5)
#pragma config HFOFST = ON // HFINTOSC Fast Start-up (HFINTOSC output and ready status are not delayed by the oscillator stable status)
#pragma config T3CMX = PORTC0 // Timer3 Clock input mux bit (T3CKI is on RC0)
#pragma config P2BMX = PORTB5 // ECCP2 B output mux bit (P2B is on RB5)
#pragma config MCLRE = INTMCLR // MCLR Pin Enable bit (RE3 input pin enabled; MCLR disabled)

// CONFIG4L
#pragma config STVREN = OFF // Stack Full/Underflow Reset Enable bit (Stack full/underflow will not cause Reset)
#pragma config LVP = ON // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled if MCLRE is also 1)
#pragma config XINST = OFF // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF // Code Protection Block 0 (Block 0 (000800-003FFFh) not code-protected)
#pragma config CP1 = OFF // Code Protection Block 1 (Block 1 (004000-007FFFh) not code-protected)
#pragma config CP2 = OFF // Code Protection Block 2 (Block 2 (008000-00BFFFh) not code-protected)
#pragma config CP3 = OFF // Code Protection Block 3 (Block 3 (00C000-00FFFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF // Write Protection Block 0 (Block 0 (000800-003FFFh) not write-protected)
#pragma config WRT1 = OFF // Write Protection Block 1 (Block 1 (004000-007FFFh) not write-protected)
#pragma config WRT2 = OFF // Write Protection Block 2 (Block 2 (008000-00BFFFh) not write-protected)
#pragma config WRT3 = OFF // Write Protection Block 3 (Block 3 (00C000-00FFFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF // Table Read Protection Block 0 (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF // Table Read Protection Block 1 (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF // Table Read Protection Block 2 (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF // Table Read Protection Block 3 (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)

void _delay_x10ms(int x);
void putrs1USART(const char *data);
void putrsUSART(const char *data);
void putByteUSART(unsigned char data);
void writeByteI2C1(char i2c_add, char i2c_reg, char i2c_dat);
char readByteI2C1(char i2c_add, char i2c_reg);
char * ftoa1(float f);
void mag_cal(void);


// start ISR code
#pragma code isr = 0x08 // store the below code at address 0x08
#pragma interrupt isr

char udat[2] = {0}, gga[150] = {0}, msg[150] = {0}, new_msg = 0, new_gga = 0, gga_end = 0, c1, c2;
char *pttr = msg;
unsigned char xili;

void interrupt isr (void) {
    PIE1bits.RC1IE = 0;
// PORTB = 0;
    if(OERR1) {
        //putrsUSART("OERR");
        PORTB = 0b1100;
        CREN1 = 0;
        CREN1 = 1;
    }
    if(FERR1) {
        //putrsUSART("FERR");
        PORTB = 0b1010;
        RC1REG;
        RC1REG;
        CREN1 = 0;
        CREN1 = 1;
    }
    if(RC1IF == 1) { // if the USART receive interrupt flag has been set
        udat[0] = RC1REG;
        udat[1] = '\0';
    }
    switch (udat[0]) {
        case ('*') :
            if(new_msg) {
// PORTB = 0b0010;
                new_msg = 0;
                while(!RC1IF);
                c1 = RC1REG;
                while(!RC1IF);
                c2 = RC1REG;

                if (c1 >= '0' && c1 <= '9')
                    c1 -= 48;
                if (c1 >= 'A' && c2 <= 'F')
                    c1 -= 55;
                if (c2 >= '0' && c2 <= '9')
                    c2 -= 48;
                if (c2 >= 'A' && c2 <= 'F')
                    c2 -= 55;
                if (c1*16+c2 == xili) {
                    if (!strncmp(pttr, "GPGGA", 5)) {
                        strcpy(gga, pttr);
                        new_gga = 1;
                    }
                }
// PORTB = 0;
            }
            break;
        case ('$') :
            pttr[0] = '\0';
            new_msg = 1;
            xili = 0;
            PORTB ^= 2;
            break;
        default :
// PORTB = 0b0100;
            if (new_msg) {
                pttr = strcat(pttr, udat);
                xili ^= udat[0];
            }
// PORTB = 0;
    }
    PIE1bits.RC1IE = 1;
}

#pragma code // return to the default code section
// end ISR code

#define ADRESA_ACC (0x30)
#define ADRESA_MAG (0x3C)
#define ADRESA_GYR (0xD2)

void i2c_init(void)
{
    OpenI2C1(MASTER, SLEW_OFF);
    IdleI2C1();
    writeByteI2C1(ADRESA_ACC, 0x20, 0b00111111);
    writeByteI2C1(ADRESA_MAG, 0x00, 0b00010000);
    writeByteI2C1(ADRESA_MAG, 0x01, 0b00100000);
    writeByteI2C1(ADRESA_MAG, 0x02, 0b00000000);
    writeByteI2C1(ADRESA_GYR, 0x20, 0b00001111);
    writeByteI2C1(ADRESA_GYR, 0x2E, 0b00000000);
}
int i, j, k;
fix16_t x_y, x_z;
fix16_t roll, pitch, yaw, yaw_c, yaw_old = 0, hx, hy, c = 0;
fix16_t temp, x[5], xyy[5], xz[5], y[5], yyy[5], z[5], zz[5], x_min = 10000, x_max = -10000, y_min = 10000, y_max = -10000, z_min = 10000, z_max = -10000, x_0, y_0, z_0, modul, mm, mmm;
fix16_t x_acc, y_acc, z_acc, x_mag, y_mag, z_mag, x_gyr, y_gyr, z_gyr, x_mag_m, y_mag_m, z_mag_m, x_gyr_k = 0, y_gyr_k = 0, z_gyr_k = 0;
char x_acc_l, x_acc_h, y_acc_l, y_acc_h, z_acc_l, z_acc_h;
char x_mag_l, x_mag_h, y_mag_l, y_mag_h, z_mag_l, z_mag_h;
char x_gyr_l, x_gyr_h, y_gyr_l, y_gyr_h, z_gyr_l, z_gyr_h;
unsigned char xor = 0, cal = 0;
unsigned char *chptr;
int zar = 0, p_c;
fix16_t Q, R, xr, xr_, xr_1, xp, xp_, xp_1, xyw, xy_1, Pr, Pr_, Pr_1, Pp, Pp_, Pp_1, yr, yp, yyw, Kr, Kp;
char lat[15], lon[15], time[15], hdop[6], N, E, T, NN, EE, hd, eep;

fix16_t poly[5] = {70142,-280163,419843,-279761,69940};
//double poly[5] = {841501,-3363000,5041500,-3360000,840000};
mf16 Qrp[2][2] = {0.00002, 0, 0, 0.00002}, Rrp[2][2] = {0.001, 0, 0, 0.001}, Hrp[2][2] = {1, 0, 0, 1}, Irp[2][2] = {1, 0, 0, 1};
mf16 Prp[2][2] = {0, 0, 0, 0}, xrp[2][1], xrp_[2][1], Prp_[2][2], yrp[2][1], Srp[2][2], Krp[2][2], Frp[2][2], EKF2[2][2], EKF1[2][1];
fix16_t Qy = 0.1, Ry, Hy = 1, Iy = 1, Py = 0, xy, xy_, Py_, yy, Sy, Ky, Fy;
fix16_t tm, h;
int lt, ln;
//double 2 fp
int main() {
    OSCCONbits.IRCF = 0b111;
    OSCTUNEbits.PLLEN = 1;
    SPBRG1 = 138;
    SPBRGH1 = 0;
    TXSTA1bits.TX9 = 0;
    TXSTA1bits.TXEN = 1;
    TXSTA1bits.SYNC = 0;
    TXSTA1bits.BRGH = 1;
    RCSTA1bits.SPEN = 1;
    RCSTA1bits.RX9 = 0;
    BAUDCON1bits.BRG161 = 1;
    RCSTA1bits.CREN1 = 1;
    SSP1ADD = 0x27; //SSPADD Baud Register used to calculate I2C clock speed in MASTER mode (in this case 100Khz)
    PORTB = 0;
    TRISB = 0;
    ANSELC = 0;
    TRISC = 0;
    PORTC = 0;
    ANSELA = 0;
    TRISAbits.RA0 = 1;
    TRISCbits.TRISC3 = 1;
    TRISCbits.TRISC4 = 1;
    TRISCbits.TRISC6 = 1;
    TRISCbits.TRISC7 = 1;
    PORTB = 0b1111;
    T0CONbits.T08BIT = 0;
    T0CONbits.T0CS = 0;
    T0CONbits.PSA = 0;
    T0CONbits.T0PS = 0b111;
    TMR0H = 0xE7;
    TMR0L = 0x8A;

    //putByteUSART('A');

    i2c_init();

    // interrupts / USART interrupts configuration
    PORTB = 1;
    RCONbits.IPEN = 0; // disable interrupt priority
    PORTB = 2;
    INTCONbits.GIE = 1; // enable interrupts
    PORTB = 4;
    INTCONbits.PEIE = 1; // enable peripheral interrupts.
    PORTB = 8;
    PIE1bits.RC1IE = 1; // enable USART receive interrupt
    PORTB = 0;

    T0CONbits.TMR0ON = 1;

    unsigned char* chptr=(unsigned char*)(&x_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 0*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&y_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 1*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&z_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 2*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&x_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 3*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&y_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 4*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&z_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 5*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&modul);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 6*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&x_gyr_k);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 7*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&y_gyr_k);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 8*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&z_gyr_k);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 9*4 + i);
    }


    x_y = (x_max - x_0) / (y_max - y_0);
    x_z = (x_max - x_0) / (z_max - z_0);

    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(y_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(z_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_y)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_z)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(y_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(z_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(modul)));
    putrsUSART("\r\n");
    //ftoa1(fp2double(x_0))
    x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
    x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
    y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
    y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
    z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
    z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

    x_mag_h = readByteI2C1(ADRESA_MAG, 0x03);
    x_mag_l = readByteI2C1(ADRESA_MAG, 0x04);
    z_mag_h = readByteI2C1(ADRESA_MAG, 0x05);
    z_mag_l = readByteI2C1(ADRESA_MAG, 0x06);
    y_mag_h = readByteI2C1(ADRESA_MAG, 0x07);
    y_mag_l = readByteI2C1(ADRESA_MAG, 0x08);

    x_acc = fix16_from_dbl((x_acc_h << 8) + x_acc_l); // fp((x_acc_h << 8) + x_acc_l)
    y_acc = fix16_from_dbl((y_acc_h << 8) + y_acc_l);
    z_acc = fix16_from_dbl((z_acc_h << 8) + z_acc_l);

    x_mag = fix16_from_dbl((x_mag_h << 8) + x_mag_l);
    y_mag = fix16_from_dbl((y_mag_h << 8) + y_mag_l);
    z_mag = fix16_from_dbl((z_mag_h << 8) + z_mag_l);

    x_mag /= fix16_from_dbl(1055);
    y_mag /= fix16_from_dbl(950);
    z_mag /= fix16_from_dbl(1055);

    x_mag -= x_0;
    y_mag -= y_0;
    z_mag -= z_0;
    y_mag *= x_y;
    z_mag *= x_z;

    x_acc /= fix16_from_dbl(-16384);
    y_acc /= fix16_from_dbl(-16384);
    z_acc /= fix16_from_dbl(-16384);


    putrsUSART(ftoa1(fix16_to_dbl(x_mag)));
    putrsUSART(ftoa1(fix16_to_dbl(y_mag)));
    putrsUSART(ftoa1(fix16_to_dbl(z_mag)));
    roll = fix16_atan2(y_acc, z_acc); //fp_fix16_atan2
    pitch = fix16_atan2(x_acc, fix16_sqrt(y_acc * y_acc + z_acc * z_acc));
    pitch *= (-1);
    putrsUSART(ftoa1(fix16_to_dbl(roll)));
    putrsUSART(ftoa1(fix16_to_dbl(pitch)));
    hx = x_mag*fix16_cos(pitch)+y_mag*sin(pitch)*fix16_sin(roll)+z_mag*fix16_cos(roll)*fix16_sin(pitch);
    hy = y_mag*fix16_cos(roll)-z_mag*fix16_sin(roll);
    yaw = fix16_atan2(-hy, hx);

    xrp[0][0] = roll;
    xrp[1][0] = pitch;
    xy = yaw;

    //_delay_x10ms(1000);

    while(1) {

        if(INTCONbits.TMR0IF) {
            INTCONbits.TMR0IF = 0;
            PORTB ^= 1;
            TMR0H = 0xE7;
            TMR0L = 0x8A;
            if (PORTAbits.RA0)
                cal++;
            else
                cal = 0;
            if (cal == 100) {
                cal = 0;
                mag_cal();
                INTCONbits.GIE = 1;

            }
            putrsUSART("\r\n");
            putByteUSART(TMR0L);
            putByteUSART(TMR0H);
            putrsUSART("\r\n");
            if (new_gga) {
                p_c = 0;
                zar = 0;
                N = 1;
                E = 1;
                T = 0;
                hd = 0;
                do {
                    if (zar == 1)
                        time[T++] = gga[p_c];
                    if (zar == 2)
                        lat[N++] = gga[p_c];
                    if (zar == 3) {
                        if (gga[p_c++] == 'N')
                            lat[0] = '+';
                        else
                            lat[0] = '-';
                    }
                    if (zar == 4)
                        lon[E++] = gga[p_c];
                    if (zar == 5) {
                        if (gga[p_c++] == 'E')
                            lon[0] = '+';
                        else
                            lon[0] = '-';
                    }
                    if (zar == 8)
                        hdop[hd++] = gga[p_c];
                    if (gga[p_c] == ',')
                        zar++;
                    p_c++;
                }
                while(gga[p_c]);
                gga[0] = '\0';
                new_gga = 0;
            }

// putByteUSART(TMR0L);
// putByteUSART(TMR0H);
// putrsUSART("\r\n");

            x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
            x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
            y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
            y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
            z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
            z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);


            x_mag_h = readByteI2C1(ADRESA_MAG, 0x03);
            x_mag_l = readByteI2C1(ADRESA_MAG, 0x04);
            z_mag_h = readByteI2C1(ADRESA_MAG, 0x05);
            z_mag_l = readByteI2C1(ADRESA_MAG, 0x06);
            y_mag_h = readByteI2C1(ADRESA_MAG, 0x07);
            y_mag_l = readByteI2C1(ADRESA_MAG, 0x08);


            x_gyr_l = readByteI2C1(ADRESA_GYR, 0x28);
            x_gyr_h = readByteI2C1(ADRESA_GYR, 0x29);
            y_gyr_l = readByteI2C1(ADRESA_GYR, 0x2A);
            y_gyr_h = readByteI2C1(ADRESA_GYR, 0x2B);
            z_gyr_l = readByteI2C1(ADRESA_GYR, 0x2C);
            z_gyr_h = readByteI2C1(ADRESA_GYR, 0x2D);

            x_acc = fix16_from_dbl((x_acc_h << 8) + x_acc_l); // fp((x_acc_h << 8) + x_acc_l)
            y_acc = fix16_from_dbl((y_acc_h << 8) + y_acc_l);
            z_acc = fix16_from_dbl((z_acc_h << 8) + z_acc_l);

            x_mag = fix16_from_dbl((x_mag_h << 8) + x_mag_l);
            y_mag = fix16_from_dbl((y_mag_h << 8) + y_mag_l);
            z_mag = fix16_from_dbl((z_mag_h << 8) + z_mag_l);

            x_mag /= fix16_from_dbl(1055);
            y_mag /= fix16_from_dbl(950);
            z_mag /= fix16_from_dbl(1055);
            x_mag -= x_0;
            y_mag -= y_0;
            z_mag -= z_0;
            y_mag *= x_y;
            z_mag *= x_z;

            x_gyr = fix16_from_dbl((x_gyr_h << 8) + x_gyr_l);
            y_gyr = fix16_from_dbl((y_gyr_h << 8) + y_gyr_l);
            z_gyr = fix16_from_dbl((z_gyr_h << 8) + z_gyr_l);

            x_acc /= fix16_from_dbl(-16384);
            y_acc /= fix16_from_dbl(-16384);
            z_acc /= fix16_from_dbl(-16384);


            x_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);
            y_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);
            z_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);

            x_gyr -= x_gyr_k;
            y_gyr -= y_gyr_k;
            z_gyr -= z_gyr_k;

            mm = fix16_sqrt(x_mag*x_mag+y_mag*y_mag+z_mag*z_mag);

            // putByteUSART(TMR0L);
// putByteUSART(TMR0H);
// putrsUSART("\r\n");

            mmm = mm/modul;
            //Ry = eval_poly(mmm, poly, 4);
            //Ry *= 100;

            Ry = 10;
            if (mmm < 0.9 || mmm > 1.1)
                Ry = 100;
            if (mmm < 0.8 || mmm > 1.2)
                Ry = 500000;
// Rrp[0][0] = 0.001 + fabs(x_gyr)/40;
// Rrp[1][1] = 0.001 + fabs(y_gyr)/40;

            roll = fix16_atan2(y_acc, z_acc);
            pitch = fix16_atan2(x_acc, fix16_sqrt(y_acc * y_acc + z_acc * z_acc));
            pitch *= (-1);

            xrp_[0][0] = xrp[0][0] + (x_gyr + y_gyr*fix16_sin(xrp[0][0])*tan(xrp[1][0]) + z_gyr*fix16_cos(xrp[0][0])*tan(xrp[1][0]))*0.1;
            xrp_[1][0] = xrp[1][0] + (y_gyr*fix16_cos(xrp[0][0]) - z_gyr*fix16_sin(xrp[0][0]))*0.1;
            xy_ = xy + (y_gyr*fix16_sin(xrp[0][0])/fix16_cos(xrp[1][0]) + z_gyr*fix16_cos(xrp[0][0])/fix16_cos(xrp[1][0]))*0.1;

            Frp[0][0] = 1 + (tan(xrp[1][0])*(y_gyr*fix16_cos(xrp[0][0]) - z_gyr*fix16_sin(xrp[0][0]))*0.1);
            Frp[0][1] = (y_gyr*fix16_sin(xrp[0][0]) + z_gyr*fix16_cos(xrp[0][0]))*0.1/fix16_cos(xrp[1][0])/fix16_cos(xrp[1][0]);
            Frp[1][0] = -(y_gyr*fix16_sin(xrp[0][0]) + z_gyr*fix16_cos(xrp[0][0]))*0.1;
            Frp[1][1] = 1;

            mf16_transpose(EKF2, Frp);
            mf16_mul(Prp_, Prp, EKF2);
            mf16_mul(EKF2, Frp, Prp_);
            mf16_add(Prp_, EKF2, Qrp);

            yrp[0][0] = ((roll - xrp_[0][0]));
            yrp[1][0] = ((pitch - xrp_[1][0]));

            mf16_transpose(EKF2, Hrp);
            mf16_mul(Srp, Prp_, EKF2);
            mf16_mul(EKF2, Hrp, Srp);
            mf16_add( Srp, EKF2, Rrp);

            //i2x2(Srp, Krp);
            mf16_mul(EKF2, Hrp, Krp);
            mf16_mul(Krp, Prp_, EKF2);

            mf16_mul(EKF1, Krp, yrp);
            mf16_add(xrp, xrp_, EKF1);

            mf16_mul(Prp, Krp, Hrp,);
            mf16_sub(EKF2, Irp, Prp);
            mf16_mul(Prp, EKF2, Prp_);

            Fy = 1;

            Py_ = Fy*Py*Fy + Qy; //Fy transponirano

            hx = x_mag * fix16_cos(xrp[1][0]) + y_mag * fix16_sin(xrp[1][0]) * fix16_sin(xrp[0][0])+ z_mag * fix16_cos(xrp[0][0]) * fix16_sin (xrp[1][0]);
            hy = y_mag * fix16_cos(xrp[0][0]) - z_mag * fix16_sin(xrp[0][0]);
            yaw = fix16_atan2(-hy,hx);

            yy = fix16_asin(fix16_sin(yaw-xy));

            Sy = Hy*Py_*Hy + Ry; //Hy transponirano

            Ky = Py_*Hy/Sy;

            xy = xy_ + Ky*yy;

            Py = (Iy - Ky*Hy)*Py_;

            putByteUSART(0xFF);
            putByteUSART(0xFF);
            putByteUSART(0xFF);
            putByteUSART(0xFF);
// putByteUSART(TMR0L);
// putByteUSART(TMR0H);
// putrsUSART("\r\n");

// putrsUSART("\r\nu = ");
            time[T-1] = '\0';
// putrsUSART(time);
// tm = atof(time);
// chptr = (unsigned char *) &tm;
// for(i=0; i<4;i++) {
// xor ^= *chptr;
// putByteUSART(*chptr++);
// }
            time[0] = '\0';
// putrsUSART("\r\nlt = ");
            lat[N-1] = '\0';
// putrsUSART(lat);
            lt = atoi(lat);
            chptr = (unsigned char *) &lt;
            for(i=0; i<2;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            lt = atoi(&lat[N-5]);
            chptr = (unsigned char *) &lt;
            for(i=0; i<2;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            lat[0] = '\0';
// putrsUSART("\r\nln = ");
            lon[E-1] = '\0';
// putrsUSART(lon);
            ln = atoi(lon);
            chptr = (unsigned char *) &ln;
            for(i=0; i<2;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            ln = atoi(&lon[E-5]);
            chptr = (unsigned char *) &ln;
            for(i=0; i<2;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            lon[0] = '\0';
// putrsUSART("\r\nh = ");
            hdop[hd-1] = '\0';
// putrsUSART(hdop);
            h = atof(hdop);
            chptr = (unsigned char *) &h;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            hdop[0] = '\0';

            double x_acc_temp = fix16_to_dbl(x_acc);
            chptr = (unsigned char *) &x_acc_temp; //pomocna varijabla fp2double
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double y_acc_temp = fix16_to_dbl(y_acc);
            chptr = (unsigned char *) &y_acc_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double z_acc_temp = fix16_to_dbl(z_acc);
            chptr = (unsigned char *) &z_acc_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double x_gyr_temp = fix16_to_dbl(x_gyr);
            chptr = (unsigned char *) &x_gyr_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double y_gyr_temp = fix16_to_dbl(y_gyr);
            chptr = (unsigned char *) &y_gyr_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double z_gyr_temp = fix16_to_dbl(z_gyr);
            chptr = (unsigned char *) &z_gyr_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double x_mag_temp = fix16_to_dbl(x_mag);
            chptr = (unsigned char *) &x_mag_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double y_mag_temp = fix16_to_dbl(y_mag);
            chptr = (unsigned char *) &y_mag_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double z_mag_temp = fix16_to_dbl(z_mag);
            chptr = (unsigned char *) &z_mag_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            chptr = (unsigned char *) &xrp[0][0]; //rad
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            chptr = (unsigned char *) &xrp[1][0]; //rad
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double xy_temp = fix16_to_dbl(xy);
            chptr = (unsigned char *) &xy_temp; //rad
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
// chptr = (unsigned char *) &modul;
// for(i=0; i<4;i++) {
// xor ^= *chptr;
// putByteUSART(*chptr++);
// }
            double Ry_temp = fix16_to_dbl(Ry);
            chptr = (unsigned char *) &Ry_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double mmm_temp = fix16_to_dbl(mmm);
            chptr = (unsigned char *) &mmm_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            double mm_temp = fix16_to_dbl(mm);
            chptr = (unsigned char *) &mm_temp;
            for(i=0; i<4;i++) {
                xor ^= *chptr;
                putByteUSART(*chptr++);
            }
            putByteUSART(xor);
            xor = 0;
// putrsUSART("\r\n");
// putByteUSART(TMR0L);
// putByteUSART(TMR0H);
// putrsUSART("\r\n");





        }
        ClrWdt();
    }
}
void _delay_x10ms(int x) {
    int ff;
    for (ff = 0;ff < x;ff++)
        __delay_ms(10);
}

void putrs1USART(const char *data) {
    do {
        while(!TX1IF);
        TXREG1 = *data;
    }
    while( *data++ );
}

void putrsUSART(const char *data) {
    do {
        while(!(TXSTA1 & 0x02));
        TXREG1 = *data;
    }
    while( *data++ && *data !='\0');
}

void putByteUSART(unsigned char data) {
    while(!(TXSTA1 & 0x02));
    TXREG1 = data;
}

void writeByteI2C1(char i2c_add, char i2c_reg, char i2c_dat) {
    StartI2C1();
    IdleI2C1();
    do {
        putcI2C1( i2c_add ); //send address
        IdleI2C1();
    } while(SSP1CON2bits.ACKSTAT);
    do {
        putcI2C1(i2c_reg); //send register
        IdleI2C1();
    } while(SSP1CON2bits.ACKSTAT);
    do {
        putcI2C1(i2c_dat); //send data
        IdleI2C1();
    } while(SSP1CON2bits.ACKSTAT);
    StopI2C1();
    IdleI2C1();
}

char readByteI2C1(char i2c_add, char i2c_reg) {
    char data;
    StartI2C1(); // Start condition
    IdleI2C1();
    do {
        WriteI2C1(i2c_add); // addresses the chip with a read bit
        IdleI2C1();
    } while (SSP1CON2bits.ACKSTAT);
    do {
        WriteI2C1(i2c_reg); // addresses the chip with a read bit
        IdleI2C1();
    } while (SSP1CON2bits.ACKSTAT);
    RestartI2C1();
    WriteI2C1(i2c_add + 1);
    IdleI2C1();
    data = ReadI2C1(); // read the value from the RTC and store in result
    NotAckI2C1();
    StopI2C1(); // Stop condition I2C on bus
    IdleI2C1();
    return data;
}

char * ftoa2(float f)
{
static char	buf[17],lbuf[11],rbuf[7];
char *	cp = buf;
        unsigned long	l, rem;
        char i,sum;

if(f < 0) {
            putByteUSART('-');
            f = -f;
        }
        else {
            putByteUSART('+');
        }
l = (unsigned long)f;
f -= (float)l;
rem = (unsigned long)(f * 1e6);
        ltoa(rbuf, rem, 10);
        ltoa(lbuf, l, 10);
        putrsUSART(lbuf);
        putByteUSART('.');
        sum = 6-strlen(rbuf);
        for(i=0;i<(sum);i++)
            putByteUSART('0');
        putrsUSART(rbuf);
// sprintf(cp, "%03lu.%6.6lu", l, rem);
// sprintf(cp, "%.6f", f);
        buf[0] = 0;
        return buf;
}

char * ftoa1(float f)
{
static char	buf[17];
char *	cp = buf;
unsigned long	l, rem;

if(f < 0) {
            *cp++ = '-';
            f = -f;
        }
        else {
            *cp++ = '+';

        }
l = (unsigned long)f;
f -= (float)l;
rem = (unsigned long)(f * 1e6);
sprintf(cp, "%lu.%6.6lu", l, rem);
return buf;
}

void mag_cal(void) {
    INTCONbits.GIE = 0;

    x_gyr_k = 0;
    y_gyr_k = 0;
    z_gyr_k = 0;
    putrsUSART("Gyro kalibracija:");
    for(k=0; k<300; k++) {
        ClrWdt();
        _delay_x10ms(10);
        putrsUSART(ftoa1(k));
        putrsUSART("\r\n");
        x_gyr_l = readByteI2C1(ADRESA_GYR, 0x28);
        x_gyr_h = readByteI2C1(ADRESA_GYR, 0x29);
        y_gyr_l = readByteI2C1(ADRESA_GYR, 0x2A);
        y_gyr_h = readByteI2C1(ADRESA_GYR, 0x2B);
        z_gyr_l = readByteI2C1(ADRESA_GYR, 0x2C);
        z_gyr_h = readByteI2C1(ADRESA_GYR, 0x2D);

        x_gyr = fix16_from_dbl((x_gyr_h << 8) + x_gyr_l);
        y_gyr = fix16_from_dbl((y_gyr_h << 8) + y_gyr_l);
        z_gyr = fix16_from_dbl((z_gyr_h << 8) + z_gyr_l);

        x_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);
        y_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);
        z_gyr *= fix16_from_dbl(0.00875 / 180 * M_PI);

        x_gyr_k += x_gyr;
        y_gyr_k += y_gyr;
        z_gyr_k += z_gyr;
    }

    x_gyr_k /= fix16_from_dbl(300);
    y_gyr_k /= fix16_from_dbl(300);
    z_gyr_k /= fix16_from_dbl(300);

    chptr = (unsigned char *) &x_gyr_k;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 7*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &y_gyr_k;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 8*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &z_gyr_k;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 9*4 + i, *chptr++);
    }

    cal = 0;
    for(i=0; i<50; i++) {
        _delay_x10ms(10);
        if(PORTAbits.RA0)
            cal++;
        else
            cal = 0;
    }
    if (cal < 45) {
        cal = 0;
        return;
    }

    x_min = 10000;
    x_max = -10000;
    y_min = 10000;
    y_max = -10000;
    z_min = 10000;
    z_max = -10000;
    putrsUSART("\r\nKalibracija x-y ravnine magnetometra za:");
    _delay_x10ms(100);
    putrsUSART("\r\n5");
    _delay_x10ms(100);
    putrsUSART("\r\n4");
    _delay_x10ms(100);
    putrsUSART("\r\n3");
    _delay_x10ms(100);
    putrsUSART("\r\n2");
    _delay_x10ms(100);
    putrsUSART("\r\n1");
    _delay_x10ms(100);
    putrsUSART("\r\n");

    for (k=0; k<100; k++) {
        ClrWdt();
        PORTB = 0b0001;
        _delay_x10ms(5);
        x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
        x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
        y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
        y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
        z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
        z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

        x_mag_h = readByteI2C1(ADRESA_MAG, 0x03);
        x_mag_l = readByteI2C1(ADRESA_MAG, 0x04);
        z_mag_h = readByteI2C1(ADRESA_MAG, 0x05);
        z_mag_l = readByteI2C1(ADRESA_MAG, 0x06);
        y_mag_h = readByteI2C1(ADRESA_MAG, 0x07);
        y_mag_l = readByteI2C1(ADRESA_MAG, 0x08);

        x_mag = fix16_from_dbl((x_mag_h << 8) + x_mag_l);
        y_mag = fix16_from_dbl((y_mag_h << 8) + y_mag_l);
        z_mag = fix16_from_dbl((z_mag_h << 8) + z_mag_l);

        x_mag /= fix16_from_dbl(1055);
        y_mag /= fix16_from_dbl(950);
        z_mag /= fix16_from_dbl(1055);

        putrsUSART(ftoa1(fix16_to_dbl(x_mag)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(y_mag)));
        putrsUSART("\r\n");


        xyy[k%5] = x_mag;
        yyy[k%5] = y_mag;

        for (i=0; i<5; i++) {
            x[i] = xyy[i];
            y[i] = yyy[i];
        }
        // the following two loops sort the array x in ascending order
        for(i=0; i<5-1; i++) {
            for(j=i+1; j<5; j++) {
                if(x[j] < x[i]) {
                    // swap elements
                    temp = x[i];
                    x[i] = x[j];
                    x[j] = temp;
                }
            }
        }
        for(i=0; i<5-1; i++) {
            for(j=i+1; j<5; j++) {
                if(y[j] < y[i]) {
                    // swap elements
                    temp = y[i];
                    y[i] = y[j];
                    y[j] = temp;
                }
            }
        }
        if (x[2] < x_min)
            x_min = x[2];
        if (x[2] > x_max)
            x_max = x[2];
        if (y[2] < y_min)
            y_min = y[2];
        if (y[2] > y_max)
            y_max = y[2];

        putrsUSART(ftoa1(fix16_to_dbl(x_min)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(x_max)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(y_min)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(y_max)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(k));
        putrsUSART("\r\n\r\n");
        PORTB = 0b0010;
        _delay_x10ms(5);
    }

    x_0 = x_max - ((x_max - x_min) / 2);
    y_0 = y_max - ((y_max - y_min) / 2);
    x_y = (x_max - x_0) / (y_max - y_0);


    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(y_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(y_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_y)));
    putrsUSART("\r\n");

    cal = 0;
    for(i=0; i<50; i++) {
        _delay_x10ms(10);
        if(PORTAbits.RA0)
            cal++;
        else
            cal = 0;
        if(cal > 45) {
            cal = 0;
            return;
        }

    }

    chptr = (unsigned char *) &x_0;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 0*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &y_0;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 1*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &x_max;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 3*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &y_max;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 4*4 + i, *chptr++);
    }

    x_min = 10000;
    x_max = -10000;

    while(1) {
        ClrWdt();
        x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
        x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
        y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
        y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
        z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
        z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

        x_acc = fix16_from_dbl((x_acc_h << 8) + x_acc_l);
        y_acc = fix16_from_dbl((y_acc_h << 8) + y_acc_l);
        z_acc = fix16_from_dbl((z_acc_h << 8) + z_acc_l);

        x_acc /= fix16_from_dbl(-16384);
        y_acc /= fix16_from_dbl(-16384);
        z_acc /= fix16_from_dbl(-16384);

        roll = fix16_atan2(y_acc, z_acc);
        pitch = fix16_atan2(x_acc, fix16_sqrt(y_acc * y_acc + z_acc * z_acc));
        pitch *= (-1);

        putByteUSART(0xFF);
        putByteUSART(0xFF);
        putByteUSART(0xFF);
        putByteUSART(0xFF);

        for(i=0; i<48;i++) {
            xor ^= 0;
            putByteUSART(0);
        }

        double roll_temp = fix16_to_dbl(roll);
        chptr = (unsigned char *) &roll_temp;
        for(i=0; i<4;i++) {
            xor ^= *chptr;
            putByteUSART(*chptr++);
        }
        double pitch_temp = fix16_to_dbl(pitch);
        chptr = (unsigned char *) &pitch_temp;
        for(i=0; i<4;i++) {
            xor ^= *chptr;
            putByteUSART(*chptr++);
        }

        for(i=0; i<16;i++) {
            xor ^= 0;
            putByteUSART(0);
        }

        putByteUSART(xor);
        xor = 0;

        _delay_x10ms(10);
        if (PORTAbits.RA0)
                cal++;
            else
                cal = 0;
            if (cal == 10) {
                cal = 0;
                break;
            }
    }

    putrsUSART("\r\nKalibracija x-z ravnine magnetometra za:");
    _delay_x10ms(100);
    putrsUSART("\r\n5");
    _delay_x10ms(100);
    putrsUSART("\r\n4");
    _delay_x10ms(100);
    putrsUSART("\r\n3");
    _delay_x10ms(100);
    putrsUSART("\r\n2");
    _delay_x10ms(100);
    putrsUSART("\r\n1");
    _delay_x10ms(100);
    putrsUSART("\r\n");

    for (k=0; k<100; k++) {
        ClrWdt();
        PORTB = 0b0100;
        _delay_x10ms(5);
        x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
        x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
        y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
        y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
        z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
        z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

        x_mag_h = readByteI2C1(ADRESA_MAG, 0x03);
        x_mag_l = readByteI2C1(ADRESA_MAG, 0x04);
        z_mag_h = readByteI2C1(ADRESA_MAG, 0x05);
        z_mag_l = readByteI2C1(ADRESA_MAG, 0x06);
        y_mag_h = readByteI2C1(ADRESA_MAG, 0x07);
        y_mag_l = readByteI2C1(ADRESA_MAG, 0x08);

        x_mag = (x_mag_h << 8) + x_mag_l;
        y_mag = (y_mag_h << 8) + y_mag_l;
        z_mag = (z_mag_h << 8) + z_mag_l;

        x_mag /= 1055;
        y_mag /= 950;
        z_mag /= 1055;

        putrsUSART(ftoa1(fix16_to_dbl(x_mag)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(z_mag)));
        putrsUSART("\r\n");

        xz[k%5] = x_mag;
        zz[k%5] = z_mag;

        for (i=0; i<5; i++) {
            x[i] = xz[i];
            z[i] = zz[i];
        }
        // the following two loops sort the array x in ascending order
        for(i=0; i<5-1; i++) {
            for(j=i+1; j<5; j++) {
                if(x[j] < x[i]) {
                    // swap elements
                    temp = x[i];
                    x[i] = x[j];
                    x[j] = temp;
                }
            }
        }
        for(i=0; i<5-1; i++) {
            for(j=i+1; j<5; j++) {
                if(z[j] < z[i]) {
                    // swap elements
                    temp = z[i];
                    z[i] = z[j];
                    z[j] = temp;
                }
            }
        }
        if (x[2] < x_min)
            x_min = x[2];
        if (x[2] > x_max)
            x_max = x[2];
        if (z[2] < z_min)
            z_min = z[2];
        if (z[2] > z_max)
            z_max = z[2];

        putrsUSART(ftoa1(fix16_to_dbl(x_min)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(x_max)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(z_min)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(fix16_to_dbl(z_max)));
        putrsUSART("\r\n");
        putrsUSART(ftoa1(k));
        putrsUSART("\r\n\r\n");
        PORTB = 0b1000;
        _delay_x10ms(5);
    }

// x_0 = x_max - ((x_max - x_min) / 2);
    z_0 = z_max - ((z_max - z_min) / 2);
    x_z = (x_max - x_0) / (z_max - z_0);

    putrsUSART(ftoa1(fix16_to_dbl(z_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(z_max)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_z)));
    putrsUSART("\r\n");

    while(1) {
        ClrWdt();
        x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
        x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
        y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
        y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
        z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
        z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

        x_acc = fix16_from_dbl((x_acc_h << 8) + x_acc_l);
        y_acc = fix16_from_dbl((y_acc_h << 8) + y_acc_l);
        z_acc = fix16_from_dbl((z_acc_h << 8) + z_acc_l);

        x_acc /= fix16_from_dbl(-16384);
        y_acc /= fix16_from_dbl(-16384);
        z_acc /= fix16_from_dbl(-16384);

        roll = fix16_atan2(y_acc, z_acc);
        pitch = fix16_atan2(x_acc, fix16_sqrt(y_acc * y_acc + z_acc * z_acc));
        pitch *= (-1);

        putByteUSART(0xFF);
        putByteUSART(0xFF);
        putByteUSART(0xFF);
        putByteUSART(0xFF);

        for(i=0; i<48;i++) {
            xor ^= 0;
            putByteUSART(0);
        }

        chptr = (unsigned char *) &roll;
        for(i=0; i<4;i++) {
            xor ^= *chptr;
            putByteUSART(*chptr++);
        }
        chptr = (unsigned char *) &pitch;
        for(i=0; i<4;i++) {
            xor ^= *chptr;
            putByteUSART(*chptr++);
        }

        for(i=0; i<16;i++) {
            xor ^= 0;
            putByteUSART(0);
        }

        putByteUSART(xor);
        xor = 0;

        _delay_x10ms(10);
        if (PORTAbits.RA0)
                cal++;
            else
                cal = 0;
            if (cal == 10) {
                cal = 0;
                break;
            }
    }

    putrsUSART("\r\nKalibracija modula magnetometra za:");
    _delay_x10ms(100);
    putrsUSART("\r\n5");
    _delay_x10ms(100);
    putrsUSART("\r\n4");
    _delay_x10ms(100);
    putrsUSART("\r\n3");
    _delay_x10ms(100);
    putrsUSART("\r\n2");
    _delay_x10ms(100);
    putrsUSART("\r\n1");
    _delay_x10ms(100);
    putrsUSART("\r\n");

    x_mag_m = 0;
    y_mag_m = 0;
    z_mag_m = 0;

    for (k=0; k<50; k++) {
        ClrWdt();
        PORTB = 0b0100;
        _delay_x10ms(5);
        x_acc_l = readByteI2C1(ADRESA_ACC, 0x28);
        x_acc_h = readByteI2C1(ADRESA_ACC, 0x29);
        y_acc_l = readByteI2C1(ADRESA_ACC, 0x2A);
        y_acc_h = readByteI2C1(ADRESA_ACC, 0x2B);
        z_acc_l = readByteI2C1(ADRESA_ACC, 0x2C);
        z_acc_h = readByteI2C1(ADRESA_ACC, 0x2D);

        x_mag_h = readByteI2C1(ADRESA_MAG, 0x03);
        x_mag_l = readByteI2C1(ADRESA_MAG, 0x04);
        z_mag_h = readByteI2C1(ADRESA_MAG, 0x05);
        z_mag_l = readByteI2C1(ADRESA_MAG, 0x06);
        y_mag_h = readByteI2C1(ADRESA_MAG, 0x07);
        y_mag_l = readByteI2C1(ADRESA_MAG, 0x08);

        x_mag = fix16_from_dbl((x_mag_h << 8) + x_mag_l);
        y_mag = fix16_from_dbl((y_mag_h << 8) + y_mag_l);
        z_mag = fix16_from_dbl((z_mag_h << 8) + z_mag_l);

        x_mag /= fix16_from_dbl(1055);
        y_mag /= fix16_from_dbl(950);
        z_mag /= fix16_from_dbl(1055);

        x_mag -= x_0;
        y_mag -= y_0;
        z_mag -= z_0;
        y_mag *= x_y;
        z_mag *= x_z;

        x_mag_m += x_mag;
        y_mag_m += y_mag;
        z_mag_m += z_mag;

        PORTB = 0b0010;
        _delay_x10ms(5);
    }

    x_mag_m /= 50;
    y_mag_m /= 50;
    z_mag_m /= 50;
    modul = fix16_sqrt(x_mag_m*x_mag_m+y_mag_m*y_mag_m+z_mag_m*z_mag_m);


    putrsUSART(ftoa1(modul));
    putrsUSART("\r\n");
    _delay_x10ms(200);

    chptr = (unsigned char *) &z_0;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 2*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &z_max;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 5*4 + i, *chptr++);
    }
    chptr = (unsigned char *) &modul;
    for(i=0; i<4;i++) {
        Busy_eep();
        Write_b_eep(1 + 6*4 + i, *chptr++);
    }

    putrsUSART("\r\n");
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(z_0)));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(fix16_to_dbl(x_z)));
    putrsUSART("\r\n");
    putrsUSART("\r\n");
    _delay_x10ms(200);

    unsigned char* chptr=(unsigned char*)(&x_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 0*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&y_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 1*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&z_0);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 2*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&x_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 3*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&y_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 4*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&z_max);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 5*4 + i);
    }
    unsigned char* chptr=(unsigned char*)(&modul);
    for(i=0; i<4;i++) {
        Busy_eep();
        chptr[i] = Read_b_eep(1 + 6*4 + i);
    }


    x_y = (x_max - x_0) / (y_max - y_0);
    x_z = (x_max - x_0) / (z_max - z_0);

    putrsUSART("\r\n");
    putrsUSART(ftoa1(x_0));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(y_0));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(z_0));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(x_y));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(x_z));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(x_max));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(y_max));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(z_max));
    putrsUSART("\r\n");
    putrsUSART(ftoa1(modul));
    putrsUSART("\r\n");
    _delay_x10ms(500);
// modul /= 1000;
}